//Purpose:
//Queries the Time Slots Object and creates ad-hoc events for the unavailable
//calendar to reflect this information

//Action Type: Custom App Action - After Events Rendered
//Prevent Default Action: Yes

//More info on custom event actions here:
//https://docs.dayback.com/article/140-custom-app-actions

//check params.data object for how this is called, so the code only executes once
if (params.data.fromViewStateChange || params.data.fromRefresh || params.data.fromFilterChange) {
  callTimeSlots();
}

function callTimeSlots() {
  //retrieve our canvas client object for authentication
  var client = fbk.client();

  //retrieve our canvas context object for REST links/urls
  var context = fbk.context();

  //retrieve the query URL from context
  var url = context.links.queryUrl;

  //SOQL Select Statement Fields;
  var select = 'SELECT+DayOfWeek,StartTime,EndTime,OperatingHoursId+FROM+TimeSlot+';

  //SOQL Order By Clause
  var orderBy = 'ORDER BY OperatingHoursId,DayOfWeek,StartTime';

  //SOQL query
  var query = select + '+' + orderBy;

  //final URL for GET
  var finalUrl = url + '?q=' + query;

  //build settings object for Ajax call to Salesforce
  var settings = {};
  settings.client = client;
  settings.contentType = 'application/json';
  settings.success = processResult;

  //Use canvas function to query
  Sfdc.canvas.client.ajax(finalUrl, settings);
}

function processResult(data) {
  if (data && data.status === 200) {
    //locate the unavailable schedule that's going to be appended
    var schedules = seedcodeCalendar.get("schedules");
    var schedule = false;
    for (var i = 0; i < schedules.length; i++) {
      if (schedules[i].isUnavailable) {
        schedule = schedules[i];
        break;
      }
    }
    if (!schedule) {
      return false;
    }
    var records = data.payload.records;
    //retrieve operating hours map loaded On Resources Fetched
    var operatingHours = seedcodeCalendar.get("operatingHours");
    //get the current view state and selected resources
    var view = seedcodeCalendar.get("view");
    var resources = seedcodeCalendar.get("resources");
    //determine selected resources and write to new array
    var selectedResources = [];
    var selectedHash = {};
    var names = [];
    for (var r = 0; r < resources.length; r++) {
      if (!resources[r].isFolder && resources[r].status && resources[r].status.selected && !selectedHash[resources[r].name]) {
        selectedHash[resources[r].name] = true;
        selectedResources.push(resources[r]);
        names.push(resources[r].name);
      }
    }
    //initialize array of new ad-hoc objects
    var result = [];
    //cycle through dates;
    var thisDate = view.start.clone();
    while (thisDate.isBefore(view.end)) {
      for (var s = 0; s < selectedResources.length; s++) {
        var thisResource = selectedResources[s].name;
        var thisDay = thisDate.format("dddd");
        var times = getTimes(thisDate, thisResource);
        var block = {};
        var allDay = {};

        //create final start and end
        var dayStart = moment(thisDate);
        var dayEnd = dayStart.clone().add("hours", 24);

        if (!times) {
          //no hours for this day;
          //whole day is unavailable
          //no hours for this day;
          allDay.contactID = [];
          allDay.projectID = [];
          allDay.contactName = [];
          allDay.projectName = [];
          allDay.status = [];
          allDay.resource = [thisResource];
          allDay.start = $.fullCalendar.moment(dayStart.format());
          allDay.end = $.fullCalendar.moment(dayEnd.format());
          allDay.title = '';
          allDay.titleEdit = '';
          allDay.location = '';
          allDay.eventID = utilities.generateUID();
          allDay.description = '**OperatingHours';
          allDay.eventSource = schedule.id;
          allDay.schedule = schedule;
          result.push(allDay);
        }
        else {
          for (var t = 0; t < times.length; t++) {

            var block = {};
            var thisBlockStart = current;
            var thisBlockEnd = times[t].start;

            block.contactID = [];
            block.projectID = [];
            block.contactName = [];
            block.projectName = [];
            block.status = [];
            block.resource = [thisResource];
            block.start = $.fullCalendar.moment(thisBlockStart.format());
            block.end = $.fullCalendar.moment(thisBlockEnd.format());
            block.title = '';
            block.titleEdit = '';
            block.location = '';
            block.eventID = utilities.generateUID();
            block.description = '**OperatingHours';
            block.eventSource = schedule.id;
            block.schedule = schedule;
            result.push(block);
            current = moment(times[t].end);
          }
          var finalBlock = {};
          finalBlock.contactID = [];
          finalBlock.projectID = [];
          finalBlock.contactName = [];
          finalBlock.projectName = [];
          finalBlock.status = [];
          finalBlock.resource = [thisResource];
          finalBlock.start = $.fullCalendar.moment(current.format());
          finalBlock.end = $.fullCalendar.moment(dayEnd.format());
          finalBlock.title = '';
          finalBlock.titleEdit = '';
          finalBlock.location = '';
          finalBlock.eventID = utilities.generateUID();
          finalBlock.description = '**OperatingHours';
          finalBlock.eventSource = schedule.id;
          finalBlock.schedule = schedule;
          result.push(finalBlock);
        }
      }
      thisDate.add(1, "days");
    }


    //push result to DOM
    dbk.addEvents(result);

    //make sure these sources are always selected
    location.hash = "#/?source=Unavailable";

  }

  function getTimes(day, resource) {
    var dayOfWeek = day.format('dddd');
    var operatingHoursId = operatingHours[resource];
    var result = [];
    for (var r = 0; r < records.length; r++) {
      if (records[r].DayOfWeek === dayOfWeek && records[r].OperatingHoursId === operatingHoursId) {
        var startTime = records[r].StartTime.replace('Z', '');
        var endTime = records[r].EndTime.replace('Z', '');
        var start = moment(day.format('YYYY-MM-DD') + ' ' + startTime);
        var end = moment(day.format('YYYY-MM-DD') + ' ' + endTime);
        var block = { 'start': start, 'end': end };
        result.push(block);
        //return {'start':start,'end':end};
      }
    }
    if (result.length > 0) {
      return result;
    }
    else {
      return false;
    }

  }

}